#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import rclpy                                     # ROS2 Python接口库
from rclpy.node import Node                      # ROS2 节点类
from geometry_msgs.msg import Twist                  # 字符串消息类型

import cv2
import numpy as np
from time import time
from hobot_vio import libsrcampy
from hobot_dnn import pyeasy_dnn as dnn

class ResNetControlNode(Node):
    
    def __init__(self, name):
        super().__init__(name)                                          # ROS2节点父类初始化
        self.cmd_vel_pub = self.create_publisher(Twist, "/cmd_vel", 10)   # 创建发布者对象（消息类型、话题名、队列长度）
        self.timer = self.create_timer(0.066, self.timer_callback)  # 创建一个定时器（单位为秒的周期，定时执行的回调函数）
        self.twist = Twist()

        ## 创建VPS和camera等对象，开始图像采集
        # camera对象
        self.cap_mipi = libsrcampy.Camera()
        self.height1, self.width1 = 512, 512
        self.height2, self.width2 = 720, 1280
        self.height, self.width = 1440, 2560
        ret = self.cap_mipi.open_cam(0, -1, 30, [self.width1, self.width2, self.width], [self.height1, self.height2, self.height])
        print("cap_mipi.open_cam State: ", end="")
        print(ret)
        # vps对象
        self.vps = libsrcampy.Camera()
        ret = self.vps.open_vps(1, 1, 512, 512, 224, 224)
        print("vps State: ", end="")
        print(ret)
        ## 导入ResNet18模型（绝对路径）
        self.models = dnn.load('/root/00_com_ws/DataSet3_train1.bin')
        ## PID相关参数
        self.Kp = 2.0
        self.Kd = 3.0
        self.Target_value = 100.0   # 设定值初始化
        self.last_Err = 0.0         # 上一个误差值初始化
        self.total_Err = 0.0        # 误差累加初始化
        self.output = 0.0           # PID输出初始化


    def timer_callback(self):  # 创建定时器周期执行的回调函数
        # 获取512的图
        mipi_data_512 = self.cap_mipi.get_img(2, self.width1, self.height1)

        # 转化为224_img
        self.vps.set_img(mipi_data_512)
        mipi_data_224 = self.vps.get_img(2, 224, 224)
        outputs = self.models[0].forward(np.frombuffer(mipi_data_224, dtype=np.uint8))
        outputs = outputs[0].buffer
        x, y = int(224*outputs[0][0][0][0]), int(224*outputs[0][1][0][0]) 

        ## PID角速度控制目标计算
        self.Error = self.Target_value - 1.0*x        # 计算偏差
        self.total_Err = self.total_Err + self.Error    # 偏差累加
        self.output = self.Kp * self.Error + self.Kd * (self.Error - self.last_Err)   # PID运算
        self.last_Err = self.Error     # 将本次偏差赋给上次一偏差



        print("( %03d, %03d )  output = %03d"%(x,y,int(self.output)))
        self.twist.linear.x = 0.2
        self.twist.angular.z = self.output/114.0
        self.cmd_vel_pub.publish(self.twist)
        
def main(args=None):                                 # ROS2节点主入口main函数
    rclpy.init(args=args)                            # ROS2 Python接口初始化
    node = ResNetControlNode("resnet_control_node")     # 创建ROS2节点对象并进行初始化
    rclpy.spin(node)                                 # 循环等待ROS2退出
    node.destroy_node()                              # 销毁节点对象
    rclpy.shutdown()                                 # 关闭ROS2 Python接口

# 偷个懒，省得编译
if __name__ == "__main__":
    main()